    subroutine EXTMATRIX(motion,luminance,intra,inter)

! input        - motion    integer*4           value 0 - 32 (0=low, 32=high)
! input        - luminance integer*4           averaged GOP luminance per pixel (0 - 255)
! input/output - intra     integer*2 array     intra matrix (64 values, row order)
! input/output - inter     integer*2 array     inter matrix (64 values, row order)

!DEC$ ATTRIBUTES DLLEXPORT:: EXTMATRIX         ! export routine

    integer*4 motion,luminance
    integer*2 intra(64),inter(64)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                     !
! Example code for adaptive matrix DLL creation.      !
!                                                     !
! DLL: matrix.dll, should be in the same directory    !
! as the HCenc executable.                            !
!                                                     !
! This Fortran example adapts the intra and inter     !
! matrices dependent of source motion and luminance.  !
! If the *ADAPTIVEMATRIX command is present,          !
! this routine is called at every GOP during the      !
! first pass so the matrices can change at every GOP. !
!                                                     !
! Of course you can create any matrix you want, to be !
! sure matrix values are valid, HCenc will limit all  !
! values between 8 and 255 and set intra(1) to 8,     !
! just like the last 3 lines of code in this example. !
!                                                     !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    real*4 lum1(0:64) &
	/0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.85, & !  0 - 16
	      0.85,0.85,0.85,0.85,0.85,0.85,0.85,0.86,0.86,0.86,0.87,0.87,0.88,0.88,0.89,0.89, & ! 17 - 32
	      0.90,0.90,0.91,0.91,0.92,0.92,0.93,0.93,0.94,0.94,0.95,0.96,0.97,0.98,0.98,0.99, & ! 33 - 48
	      0.99,0.99,0.99,1.00,1.00,1.00,1.00,1.00,1.00,1.00,1.00,1.00,1.00,1.00,1.00,1.00/   ! 49 - 64

    real*4 lum2(0:64) &
	/0.80,0.80,0.80,0.80,0.80,0.80,0.80,0.80,0.80,0.80,0.80,0.80,0.80,0.80,0.80,0.80,0.80, & !  0 - 16
	      0.80,0.80,0.80,0.80,0.80,0.80,0.80,0.80,0.81,0.81,0.82,0.82,0.83,0.83,0.84,0.84, & ! 17 - 32
	      0.85,0.85,0.86,0.86,0.87,0.87,0.88,0.88,0.89,0.90,0.91,0.92,0.92,0.93,0.93,0.94, & ! 33 - 48
	      0.94,0.95,0.95,0.96,0.96,0.97,0.97,0.98,0.98,0.98,0.99,0.99,0.99,1.00,1.00,1.00/   ! 49 - 64

    real*4 lum3(0:64) &
	/0.70,0.70,0.70,0.70,0.70,0.70,0.70,0.70,0.70,0.70,0.70,0.70,0.70,0.70,0.70,0.70,0.70, & !  0 - 16
	      0.70,0.70,0.70,0.70,0.70,0.70,0.70,0.70,0.71,0.72,0.73,0.74,0.75,0.76,0.77,0.78, & ! 17 - 32
	      0.79,0.80,0.81,0.82,0.83,0.84,0.85,0.86,0.87,0.88,0.89,0.90,0.91,0.92,0.93,0.84, & ! 33 - 48
	      0.93,0.93,0.94,0.94,0.95,0.95,0.96,0.96,0.97,0.97,0.98,0.98,0.99,0.99,1.00,1.00/   ! 49 - 64

    real*4 lum4(0:64) &
	/0.60,0.60,0.60,0.60,0.60,0.60,0.60,0.60,0.60,0.60,0.60,0.60,0.60,0.60,0.60,0.60,0.60, & !  0 - 16
	      0.60,0.60,0.60,0.60,0.60,0.60,0.60,0.60,0.61,0.62,0.63,0.64,0.65,0.66,0.67,0.68, & ! 17 - 32
	      0.69,0.70,0.71,0.72,0.73,0.74,0.75,0.76,0.77,0.78,0.79,0.80,0.81,0.82,0.83,0.84, & ! 33 - 48
	      0.85,0.86,0.87,0.88,0.89,0.90,0.91,0.92,0.93,0.94,0.95,0.96,0.97,0.98,0.99,1.00/   ! 49 - 64


! motion stuff
    if (motion <= 5) then
      a=1.-(5.-motion)/50.                     ! interpolate factor a to minimum = 0.9 for low motion
    else
      a=1.+(motion-5.)/270.                    ! interpolate factor a to maximum = 1.1 for high motion
    endif

! luminance stuff
    lu=MIN(luminance,64)                       ! adapt matrix only for luminance < 65
!   b=lum1(lu)                                 ! mild matrix adapt
!   b=lum2(lu)                                 ! moderate matrix adapt
!   b=lum3(lu)                                 ! heavy matrix adapt
    b=lum4(lu)                                 ! hit it hard...

! adapt matrix
    intra=NINT(a*b*intra)                      ! set to nearest integer for whole intra matrix
    inter=NINT(a*b*inter)                      ! set to nearest integer for whole inter matrix

! scale values into "safe values"
    intra=MAX(intra,8); intra=MIN(intra,255)   ! scale intra values, 8 <= intra <= 255
    inter=MAX(inter,8); inter=MIN(inter,255)   ! scale inter values, 8 <= inter <= 255
    intra(1)=8                                 ! first intra value should always be 8

    end
